use crate::error::Error;

use embedded_graphics::{
    prelude::*,
    mono_font::{ascii::FONT_7X13, ascii::FONT_10X20, MonoTextStyle},
    pixelcolor::BinaryColor,
    text::{Text, Alignment},
};

use esp_idf_hal::{
    self,
    prelude::Peripherals, 
    units::Hertz,
    i2c::{I2cConfig, I2cDriver, I2c} 
};

use ssd1306::{prelude::*, I2CDisplayInterface, Ssd1306};

use display_interface_i2c;


/// Enum to make clear the stage of failure of the display
#[derive(Debug)]
pub enum DisplayError {
    SetupError(display_interface::DisplayError),
    DrawingError,
    FlushError,
}

pub struct Display<'a, SIZE: DisplaySize> {
    display: Ssd1306<I2CInterface<I2cDriver<'a>>, SIZE, ssd1306::mode::BufferedGraphicsMode<SIZE>>
}

impl<'a, SIZE: DisplaySize> Display<'a, SIZE>
{
    pub fn new(i2c: I2cDriver<'a>, size: SIZE) -> Result<Display<'a, SIZE>, Error> {
        // Construct the I2C driver into a display interface
        let interface = I2CDisplayInterface::new(i2c);

        // Construct display with new anew driver
        let mut display = Ssd1306::new(interface, size, DisplayRotation::Rotate0)
            .into_buffered_graphics_mode();
        display.init().map_err(|err| Error::DisplayError(DisplayError::SetupError(err)))?;

        Ok(Display{display})
    }
}